
;;; BePlatinum/theme.jl
;;; Created by Lyonel Vincent <vincentl@ec-lyon.fr>

(let*
  (
    (font (get-font "-b&h-lucida-bold-r-normal-*-*-120-*-*-p-*-iso8859-1"))
    (t-font (get-font "-b&h-lucida-bold-r-normal-*-*-100-*-*-p-*-iso8859-1"))
    (font-colors (list (get-color "#6e6e6e") (get-color "black")))
    
    (title-width
      (lambda (w)
        (let
          ((w-width (car (window-dimensions w))))
        (max 0 (+ 20 (min (- w-width 95) (text-width (window-name w) font)))))))
          
    (update-title-width
      (lambda (w)
        (if (eq (window-get w 'current-frame-style) 'BePlatinum) 
          (rebuild-frame w))))
    
    (title-images (list (make-image "mid-title-inactive.png")
                        (make-image "mid-title-active.png")))
    (menu-images (list (make-image "menu-inactive.png")
                       (make-image "menu-active.png")
                       ))
    (close-images (list (make-image "close-inactive.png") 
                        (make-image "close-active.png")
                        ))
    (maximize-images (list (make-image "maximize-inactive.png")
                           (make-image "maximize-active.png")
                        ))
    (iconify-images (list (make-image "iconify-inactive.png")
                           (make-image "iconify-active.png")
                        ))
    (right-title-images (list (make-image "right-title-inactive.png")
                           (make-image "right-title-active.png")))
    (bottom-image (list (make-image "bottom-bar-inactive.png") (make-image "bottom-bar.png")))
    (top-image (list (make-image "top-bar-inactive.png") (make-image "top-bar.png")))
    (top-left-image (list (make-image "top-left-inactive.png") (make-image "top-left.png")))
    (top-right-image (list (make-image "top-right-inactive.png") (make-image "top-right.png")))
    (bottom-left-image (list (make-image "bottom-left-inactive.png") (make-image "bottom-left.png")))
    (bottom-right-image (list (make-image "bottom-right-inactive.png") (make-image "bottom-right.png")))
    (bottom-left-shaded-image (list (make-image "bottom-left-shaded-inactive.png") (make-image "bottom-left-shaded.png")))
    (bottom-right-shaded-image (list (make-image "bottom-right-shaded-inactive.png") (make-image "bottom-right-shaded.png")))
    (bottom-bar-shaded-image (list (make-image "bottom-bar-shaded-inactive.png") (make-image "bottom-bar-shaded.png")))
    (left-image (list (make-image "left-bar-inactive.png") (make-image "left-bar.png")))
    (right-image (list (make-image "right-bar-inactive.png") (make-image "right-bar.png")))

    (frame `(
        ((background . ,top-image)
         (top-edge . -5)
         (left-edge . 0)
         (right-edge . 0)
         (class . title))
        ((background . ,left-image)
         (left-edge . -5)
         (top-edge . 0)
         (bottom-edge . 0)
         (class . left-border))
        ((background . ,top-left-image)
         (left-edge . -5)
         (top-edge . -5)
         (class . title))
        ((background . ,top-right-image)
         (right-edge . -5)
         (top-edge . -5)
         (class . title))
        ((background . ,right-image)
         (right-edge . -5)
         (top-edge . 0)
         (bottom-edge . 0)
         (class . right-border))
        ((background . ,bottom-image)
         (left-edge . 0)
         (right-edge . 0)
         (bottom-edge . -5)
         (class . bottom-border))
        ((background . ,bottom-left-image)
         (left-edge . -5)
         (bottom-edge . -5)
         (class . bottom-left-corner))
        ((background . ,bottom-right-image)
         (right-edge . -5)
         (bottom-edge . -5)
         (class . bottom-right-corner))
        ((background . ,title-images)
         (foreground . ,font-colors)
         (font . ,font)
         (text . ,window-name)
         (top-edge . -23)
         (left-edge . 15)
         (width . ,(lambda (w) (title-width w)))
         (x-justify . center)
         (y-justify . center)
         (class . title))
        ((background . ,menu-images)
         (left-edge . -5)
         (top-edge . -23)
         (class . close-button))
        ((background . ,maximize-images)
         (left-edge . ,(lambda (w) (+ (title-width w) 15)))
         (top-edge . -23)
         (class . maximize-button))
        ((background . ,iconify-images)
         (left-edge . ,(lambda (w) (+ (title-width w) 32)))
         (top-edge . -23)
         (class . iconify-button))
      ))
    (transient-frame `(
        ((background . ,top-image)
         (top-edge . -5)
         (left-edge . 0)
         (right-edge . 0)
         (class . title))
        ((background . ,left-image)
         (left-edge . -5)
         (top-edge . 0)
         (bottom-edge . 0)
         (class . left-border))
        ((background . ,top-left-image)
         (left-edge . -5)
         (top-edge . -5)
         (class . title))
        ((background . ,top-right-image)
         (right-edge . -5)
         (top-edge . -5)
         (class . title))
        ((background . ,right-image)
         (right-edge . -5)
         (top-edge . 0)
         (bottom-edge . 0)
         (class . right-border))
        ((background . ,bottom-image)
         (left-edge . 0)
         (right-edge . 0)
         (bottom-edge . -5)
         (class . bottom-border))
        ((background . ,bottom-left-image)
         (left-edge . -5)
         (bottom-edge . -5)
         (class . bottom-left-corner))
        ((background . ,bottom-right-image)
         (right-edge . -5)
         (bottom-edge . -5)
         (class . bottom-right-corner))
        ((background . ,title-images)
         (foreground . ,font-colors)
         (font . ,font)
         (text . ,window-name)
         (top-edge . -23)
         (left-edge . 15)
         (width . ,(lambda (w) (title-width w)))
         (x-justify . center)
         (y-justify . center)
         (class . title))
        ((background . ,menu-images)
         (left-edge . -5)
         (top-edge . -23)
         (class . close-button))
        ((background . ,right-title-images)
         (left-edge . ,(lambda (w) (+ (title-width w) 15)))
         (top-edge . -23))
      ))
    (shaped-frame `(
        ((background . ,title-images)
         (foreground . ,font-colors)
         (font . ,font)
         (text . ,window-name)
         (top-edge . -23)
         (left-edge . 15)
         (width . ,(lambda (w) (title-width w)))
         (x-justify . center)
         (y-justify . center)
         (class . title))
        ((background . ,menu-images)
         (left-edge . -5)
         (top-edge . -23)
         (class . close-button))
        ((background . ,maximize-images)
         (left-edge . ,(lambda (w) (+ (title-width w) 15)))
         (top-edge . -23)
         (class . maximize-button))
        ((background . ,iconify-images)
         (left-edge . ,(lambda (w) (+ (title-width w) 32)))
         (top-edge . -23)
         (class . iconify-button))
        ((background . ,bottom-left-shaded-image)
         (left-edge . -5)
         (top-edge . -3)
         (class . title))
        ((background . ,bottom-bar-shaded-image)
         (left-edge . -4)
         (width . ,(lambda (w) (+ (title-width w) 54)))
         (top-edge . -3)
         (class . title))
        ((background . ,bottom-right-shaded-image)
         (top-edge . -4)
         (left-edge . ,(lambda (w) (+ (title-width w) 50)))
         (class . title))
      ))
    (shaped-transient-frame `(
        ((background . ,title-images)
         (foreground . ,font-colors)
         (font . ,font)
         (text . ,window-name)
         (top-edge . -23)
         (left-edge . 15)
         (width . ,(lambda (w) (title-width w)))
         (x-justify . center)
         (y-justify . center)
         (class . title))
        ((background . ,menu-images)
         (left-edge . -5)
         (top-edge . -23)
         (class . close-button))
        ((background . ,right-title-images)
         (left-edge . ,(lambda (w) (+ (title-width w) 15)))
         (top-edge . -23))
        ((background . ,bottom-left-shaded-image)
         (left-edge . -5)
         (top-edge . -3)
         (class . title))
        ((background . ,bottom-bar-shaded-image)
         (left-edge . -4)
         (width . ,(lambda (w) (+ (title-width w) 19)))
         (top-edge . -3)
         (class . title))
        ((background . ,bottom-right-shaded-image)
         (top-edge . -4)
         (left-edge . ,(lambda (w) (+ (title-width w) 15)))
         (class . title))
      ))
    )

  (add-frame-style 'BePlatinum
    (lambda (w type)
      (cond 
            ((eq type 'default) frame)
            ((eq type 'shaped) shaped-frame)
            ((eq type 'transient) transient-frame)
            ((eq type 'shaped-transient) shaped-transient-frame)
            ((eq type 'unframed) nil-frame))))
            
  (call-after-property-changed 'WM_NAME update-title-width)
)
